# Overview
The Japanese Games Translator addon lets you translate text while playing Japanese audio games.
It focuses on 2 things:

* First, to give newcomers to Japanese games the simplest experience possible to get started playing and enjoying them.
Meaning, it requires less software to install and things to configure than other solutions, and gives you step by step instructions to get started.
* Second, to provide a more automatic and seamless translation experience than is currently supported by other sets of tools.
For instance, when auto translate last spoken is enabled you can arrow through menus and immediately hear the English version of each menu item, without pressing an extra hot key.

At this point I support these games that can output unicode directly to NVDA:

* [ShadowRine FullVoice][2]
* [Bokurano Daibouken 3][3] (normal and chaos editions)
* [Bokurano Daibouken 2][6] (normal and chaos editions)
* [Bokurano Daibouken 1][7]

And these games that output to the clipboard, some of which also use Windows GUI controls, which means they require AppLocale and the Japanese keyboard:

* Brave of Cloudia, while this can output directly to NVDA, it still needs AppLocale when doing so.
* Cave of Marvels
* ShadowRine FullVoice Demo
* ShadowRine without voice acting
* Gekiransen
* MM's mini game collection

And Morokuma's text games that also require AppLocale and the Japanese keyboard:

* Alternative Magic
* Ground War 2
* Ground War 1
* Momotaro Adventure
* New Tia Story
* Tia Story 2
* Tia Story 1
* Dungeon Quest DX
* Dungeon Quest
* Mazes of Dungeon PT
* Mazes of Dungeon

It should also work in any other games created using Hot Soup Processor (HSP).
Please let me know if you test a game that is not in this list and find it works, so I can add it to this list.
If you have other Japanese games you would like it to support, contact me at support@BlindAudioGames.com with details about what the game is and how I can get started playing it.
To my knowledge, every one of these games is free, even those that have demos, which might confuse some people.

If this is your first time trying Japanese games, I strongly recommend starting with ShadowRine FullVoice and Bokurano Daibouken 3.
They are the easiest to setup, and I have included step by step instructions for them.

I started writing this addon because the better known instant translate addon stopped working due to changes Google made to stop unauthorized use of its translation service.
Since then QTranslate has become an alternative solution.
I believe this addon provides a better experience for the games it supports.
That said, the QTranslate setup can be used for things other than games, or for translating these games into languages other than English.
So it remains a good alternative.

This addon contacts my own server which then uses the paid Google Translate service.
I implement my own caching to limit the charges I receive.
This caching also helps the translations to be very fast.

You can [download the latest version of the addon here][4].
And [sign up for a free account at BlindGamers.com][1].

If you'd like to hear the addon in action, listen to this [short demo][5] that shows me using it in ShadowRine.

Please do not try to use the addon or my web service to translate text from outside these games.
I'm using the paid google translate service so it costs me money if you abuse it.

# Installing the addon in NVDA
If you have installed the non portable version of NVDA, Windows explorer will know how to install the addon just by pressing enter on it.
Alternatively you can open the NVDA menu with NVDA + N.
Then go to tools, then manage add ons.
Then tab to and press the install button and navigate to the addon in the open file dialog.

Note that Internet Explorer used to rename the file from JGT.nvda-addon to JGT.zip.
This is because NVDA addons are technically just zip files with a different file extension.
I added some extra information to my web server headers that seems to have fixed this problem, but I'm leaving this note here just in case someone still runs into this issue.

# Configuration
You must create a BlindGamers.com account in order to use the addon.
This is because addon abuse costs me money, so I need to ensure users are using the addon as it was intended.
Once you've installed the addon you can open the JGT Login dialog in NVDA preferences and enter your credentials.
Your email will be remembered, but your password is not.
You will have to reenter your password if you don't use the translation service for an extended period of time, or if you restart NVDA.

You can configure the addon by opening Preferences > JGT settings.
There are combo boxes for choosing a source and target language, though for now the only available configuration is from Japanese to English.
You can also choose whether the translated text is copied into your clipboard or not.

# Keyboard commands

* NVDA + control + shift + Y, toggles auto translate last spoken mode, this is the only key combo people need when playing a game that outputs directly to NVDA.
* NVDA + control + shift + H, toggles auto translate clipboard mode, also requires the Japanese keyboard to be active in order to work correctly.
* NVDA + shift + Y, translates the last text spoken by NVDA, not really needed when using the auto translate modes, but included anyway.

If you don't like these keyboard shortcuts, you can change them.
<pre>
Open this file in your portable NVDA folder: NVDA\userConfig\addons\JGT\globalPlugins\Ian Reed's Japanese games translator\__init__.py
Or this file if you are using an installed version of NVDA: %AppData%\nvda\addons\JGT\globalPlugins\Ian Reed's Japanese games translator\__init__.py
Then go to the bottom and find the text:
	__gestures = {
		"kb:NVDA+shift+control+h": "toggleAutoClipboard",
		"kb:NVDA+shift+y": "translateLastSpoken",
		"kb:NVDA+shift+control+y": "toggleAutoLastSpoken",
	}
</pre>
Alter it to use the keyboard commands you prefer.

# Games that can output unicode directly to NVDA
These games are the easiest to setup.
You only need to install the JGT addon and enable the auto translate last spoken mode.

## Instructions for ShadowRine FullVoice
Note that these instructions are for English players who want to do the least amount possible to start playing.
Some players may want to install AppLocale, the Japanese keyboard, and the ProTalker synthesizer.
These instructions do not cover those because they only add small benefits to the ShadowRine game play experience.

<pre>
Kyo has asked that people do not give a direct link to the game.
So I have outlined instructions for downloading the game through his site.

Here are some links to google translated portions of Kyo's site:
</pre>
[Kyo's main website, Galaxy Laboratories](https://translate.google.com/translate?sl=ja&tl=en&js=y&prev=_t&hl=en&ie=UTF-8&u=http%3A%2F%2Fwww.mm-galabo.com&edit-text=&act=url)<br>
[Translated version of the ShadowRine FullVoice download page referenced below in the Japanese instructions](https://translate.google.com/translate?sl=ja&tl=en&js=y&prev=_t&hl=en&ie=UTF-8&u=http%3A%2F%2Fwww.mm-galabo.com%2Fsr%2Fdl_srfv%2Fsrfv_dl.php&edit-text=&act=url)<br>
<pre>
For whatever reason, I could not get the game to download while using the google translate proxy to read the site in English, so below are instructions for downloading it through the Japanese version of the site.
</pre>

<!--
// instructions to download ShadowRine using google translate.
To download the game, visit:
https://translate.google.com/translate?sl=ja&tl=en&js=y&prev=_t&hl=en&ie=UTF-8&u=http%3A%2F%2Fwww.mm-galabo.com%2Fsr%2Fdl_srfv%2Fsrfv_dl.php&edit-text=&act=url
Then click the link that says "In "Furimu", you can download the "Shadow Rine FullVoice Edition ~ permanent hope ~ ver2.14""
Then click the link that says "Click here to download the "hope-of ShadowRine FullVoice Edition ~ forever."
-->

<!--// the non translated instructions. -->
<pre>
To download the game, visit:
</pre>
[http://www.mm-galabo.com/sr/dl_srfv/srfv_dl.php](http://www.mm-galabo.com/sr/dl_srfv/srfv_dl.php)
<pre>
Find and click the first link that contains the text "Shadow Rine FullVoice Edition".
At the time of this writing it is the fourth link on the page.
Then go to the ShadowRine FullVoice Edition heading.
The second link after that heading has the text "ShadowRine FullVoice Edition".
This is the link to download the game installer.
Once the installer has finished downloading, run it.
Now press enter to activate the next button whenever necessary until the installer finishes.
</pre>

<pre>
Before running the game, login to the JGT service using the JGT login dialog in NVDA preferences.
Press NVDA + control + shift + Y to turn auto translate last spoken mode on.
Then to run the game, go to C:\Program Files (x86)\GalaxyLaboratory\ShadowRine_FullVoice
Run play_sr.exe.
When you launch the game, you will be on a dialog with 5 buttons.
The first one is "Start game", so press enter.
If this is the first time you've launched the game, you'll be presented with 2 message boxes, and then a combo box that allows you to choose NVDA.
So hit enter twice, then arrow down until you find NVDA and hit enter.

If you accidentally choose something other than NVDA, you can use these steps to get the game to let you choose again.
Launch the game so you get the initial dialog with 5 buttons.
Tab twice to get to the Initialize game settings button and press enter.
Press enter again to choose Yes, then again to press OK.
You are now back on the game start button and can follow the steps above to set output to NVDA.
</pre>

<!--
When creating a new game you are asked to choose your name.
Please choose the "myself" name.  It is the first one in the list.
This will help your translations be faster and possibly save me money.
-->

Note that when voice acted dialogs appear in the game, you need to press down to have them spoken to NVDA.
Menu items and non voice acted dialogs are immediately sent to NVDA.

## ShadowRine controls
Remember to login using the JGT login dialog, then press NVDA + control + shift + Y to turn on auto translate last spoken mode.

* arrow keys, move around the field.
* X or enter, activate menu item or use interactible.
* escape, cancel or leave menu.
* C or control, use character skill.
* Z or space bar, change character.
* tab, open pause menu.
* F, open field viewer.
* S, open status menu.
* L, open load game menu.
* O, open options menu.
* comma, player display.
* period, demon display.
* slash, item display.


## Instructions for Bokurano Daibouken 3, (BK3 for short)
<pre>
Download the game from:
</pre>
[http://www.nyanchangames.com/softs/count/download.php?download=1](http://www.nyanchangames.com/softs/count/download.php?download=1)
<pre>
Run the installer when it finishes downloading.
Press enter once to install it to the default location.
It should be installed in C:\nyanchangame
The folder inside has a Japanese name, I usually rename this to BK3.
Before running the game, login to the JGT service using the JGT login dialog in NVDA preferences.
Press NVDA + control + shift + Y to turn auto translate last spoken mode on.
Run play.exe.
The game is set to automatic screen reader detection by default and so translations should work without any configuration changes.
</pre>

## BK3 controls
Remember to login using the JGT login dialog, then press NVDA + control + shift + Y to turn on auto translate last spoken mode.

* left and right, run left and right.
* control + left and right, walk left and right.
* up arrow, jump.
* tab, open the pause menu.
* enter, activate menu item, or use doors or other interactables.
* escape, cancel or leave menu.
* space bar, attack with currently equipped weapon.
* down arrow, use shield.
* H, announce health.
* Y, announce stamina.
* Q, announce your current position.
* hold G and use arrow keys, examine the field.
* O, toggle sonar, must have purchased sonar during game play.
* F, use grappling hook, acquired during game play.
* W, open equip weapon menu.
* I, open the items menu.
* C, open the collection items menu.
* S, open the shield/armor menu.
* E, open the combat log.
* D, open the object position information menu.
* L, open the level up menu.

## BK2 and BK1
I don't have any instructions for these games but testers have verified they work with NVDA directly.
The instructions and controls are likely very similar to BK3.<br>
[Download BK1 here.](http://www.nyanchangames.com/softs/nn_setup.exe)<br>
[Download BK2 here.](http://www.nyanchangames.com/softs/nn2_setup.exe)<br>
You can see Yukio's entire list of games here:<br>
[NyanchanGames.com translated to English][14]<br>
<pre>
Note when translated:
BK3 is Our large adventure Last of a loner
BK2 is Our large adventure Strikes Back, and conquest
BK1 is Ambition in our Great Adventure: darkness
</pre>
There are several other games on that page that might also work with JGT, but they have not been tested.
The other games may also require the Japanese keyboard and AppLocale to work, so I recommend trying them only after you've tried the easier to setup games.

# Games that output to the clipboard, some of which also use Windows GUI controls:
These games take a bit more effort to get setup to play.
I recommend trying these only after you've tried one of the games that output directly to NVDA.
You may also want to listen to this [demo that shows me running Alternative Magic with AppLocale and the Japanese keyboard enabled.][10]
Or this [demo that shows me using Locale Emulator to play GroundWar1.][12]

You can download most of the games in the list from this website:<br>
[List of text games, Brave of Cloudia, Cave of Marvels, and others, translated courtesy of google translate](https://translate.googleusercontent.com/translate_c?act=url&depth=1&hl=en&ie=UTF8&prev=_t&rurl=translate.google.com&sl=ja&tl=en&u=http://hirotaka2014.sakura.ne.jp/mh0406/game/&usg=ALkJrhgqmQj_Qb9i6yHntRYGS14fEVVcSw)<br>
And some more games from:<br>
[NyanchanGames.com translated to English][14]

## Installing and using the Japanese keyboard
First let me give a little explanation of why this is necessary.
If you know anything about how computers work at a low level, you may be aware that characters are really just numbers.
A character set is a mapping of numbers to language characters.
Unfortunately in the early days of computing there were many character sets, generally one for each language.
Each of these character sets mapped the same low level number to a different character.
This means that if you have text created using the Japanese character set, and you try to display it using the English character set, you get a random mess of English characters because you are using the wrong mapping.
Unicode is a character set that can represent all languages, without any of them conflicting with each other.

Whenever Japanese games copy text to the clipboard, they copy it using a non unicode Japanese character set.
So for Windows to correctly place this text on the clipboard, you must have the Japanese keyboard enabled.
Alternatively, when the games talk directly to NVDA they usually convert their text to unicode, so no Japanese keyboard is required.

Now let's get started installing the Japanese keyboard.
I only have a computer with Windows 7, so these instructions will use that, though I expect they are very similar for other Windows versions.
<pre>
Open the start menu and search for Region and Language.
Once the region and language dialog is open, press shift tab to get to the tab control.
Then arrow right until you reach the Keyboards and Languages tab.
Tab once, which places you on the Change Keyboards button, press it.
Now you are in the Text Services and Input Languages dialog.
Tab to the Add button and press it.
Now you are in a list of languages.
Press J to jump down to the Japanese option.
Press right to expand it.
Then expand the keyboard option within it.
Now press space bar on the Japanese and Microsoft IME options so that both are checked.
Press the OK button.
Press OK on the other dialogs we've opened.
</pre>

Now that the extra keyboards are installed, you can switch between language groups by pressing alt + shift, or between specific keyboards by pressing control + shift.
I find I only need to use alt + shift, so disabled the other hot key in the Advanced key settings tab of the Text Services and Input Languages dialog.

When you launch a game you will want to press alt + shift to switch to the Japanese language group.
This keyboard switching is a per application setting, so alt tabbing to another application will switch you back to the English keyboard.
If you're having trouble using NVDA or Windows hot keys, press alt + shift again to get back to your normal language, then they will work properly again.
Or just alt tab away from the game.

Remember to enable the auto translate clipboard mode before opening a game by pressing NVDA + control + shift + H.

## Installing and using AppLocale
When Japanese games use standard Windows GUI controls to display text, they need to be started using the correct locale.
There is a program called AppLocale that does this on Windows XP, Vista, and 7.
If installing on Vista or 7, you'll need to run the installer as administrator.
Apparently the official Microsoft AppLocale download page has disappeared.
Here is the [AppLocale][8] version I installed on my Windows 7 64 bit.
You may need to find a 32 bit version if that one doesn't work for you.
AppLocale does not work on Windows 10, but you can use [Locale Emulator][13] instead to solve this.

<pre>
Once AppLocale is installed, you can run it from the start menu.
After opening AppLocale, press the next button.
With the Launch an application radio button selected, tab to the browse button and press it.
Select the game executable and press OK.
Press the Next button.
Now you are in a combo box that lets you choose your language.
Press the end key to go to the very last option, which is Japanese.
It won't say anything when you are using an English voice because the text is actually in Japanese, but that's ok.
Tab to the next button and press it.
At this point you can optionally choose to create a shortcut to run the game with AppLocale in the future.
To do this, check the checkbox, then type a name in the edit box.
A sample name might be:
Alternative Magic with AppLocale
Then you can find it in the start menu in the future.
Press Finish to start the game and create the shortcut in your start menu.
Don't forget to press alt + shift to switch to the Japanese keyboard after you start the game.
</pre>

<!--
## Brave of Cloudia controls

* F2, open object position menu.
* C, open cursor browsing mode.
* tab, brings up menu, while in cursor browsing mode it switches between objects on the map.
* shift, while in cursor browsing mode it announces your current position, while in move mode it tells you what objects are nearby.
* semicolon, moves you one step closer to the nearest enemy.
-->

<!--
# CoolSoft Virtual Midi Synth and the FluidR3GM sound font
//ian fix this
unzip SFArk, copy the fluid file to the same folder.
Run sfArk in a command line with the fluid file as the first argument.
When installing virtual midi synth, check the box to make it your default midi output device.
-->

<!-- # Tyler Spivey's text game patch -->

<!-- # Are the keyboard language switching hot keys annoying you? -->

# The original Guide to playing Japanese games, updated for QTranslate
BladeStorm360 wrote the original guide to playing Japanese games.
FabiG94 recently reposted it with additional instructions for using QTranslate instead of the instant translate NVDA addon.
There is a lot of good information in the guide and it is worth reading if you want to be aware of more tools and tricks that can help you get the best experience playing Japanese games.
Here is the link to the [Guide to playing Japanese games][9]

<!-- # Translating the user guides for each game. -->

# A few notes to help you enjoy Morokuma's text games

* Enable both the auto translate last spoken and the auto translate clipboard features.
The first is needed to auto read menus, the second is needed to auto read story and NPC dialog.
* Remember to change the keyboard to Japanese by pressing alt + shift once the game is open.
* To get better sounding midi, install CoolSoft Virtual Midi Synth, and the FluidR3GM sound font.  You'll need SFArk to uncompress the sound font file.
I hope to add step by step instructions for this later.
* In the newer text games, you need to hold a modifier key when pressing enter, escape, or tab.
I usually hold shift, but control also works.
The older text games don't have this problem.
A better solution is to install [Tyler Spivey's Japanese game fix addon][11] to fix this.
You must also rename the game's executable to be play.exe if using that addon.
* Don't run ClipReader or Clipboard2Sapi at the same time as JGT's auto translate clipboard feature.
Having multiple tools announcing updates to the clipboard results in a lot of repeat messages.
* If you didn't quite hear the last speech, press NVDA + C to have the clipboard read it to you again.
Or alt tab to a text editor and paste it there for easier review.
* When alt tabbing away from the game and back, NVDA has a hard time getting focus on a control again.
In the newer text games, press control + Home to regain focus.
In the older games, when on an NPC dialog you can just press enter.
If you were in a menu you'll need to route the mouse to the current control and click it.
I use object navigation to navigate to the list view.
I do this with NVDA + numPad2.
Then I press NVDA + numPad/ to route the mouse there, and then just numPad/ to click the mouse button.
* The text is not automatically read in status screens.
Highlight one or many lines of text and then press control + C to have it automatically translated.
There is a limit on how much text can be translated at once, so highlighting everything at once won't always work.
* The JGT addon does not read the slot machines quickly enough.
I recommend using Clipboard2Sapi and the ProTalker English synthesizer to hear all the characters in Japanese, especially since some don't have an audible translation in English.
You'll want to turn off the auto translate clipboard feature so JGT is not interfering with Clipboard2Sapi.
* The games come with user manuals that sometimes have very helpful instructions.
Since JGT will not translate text outside of the games, consider using the google translate website for these.
* Consider using Tyler Spivey's text games patch that removes the long delays every time a new midi starts, removes the need to switch to the Japanese keyboard, and makes it so the older text games don't pick up your key presses when they don't have focus.
It crashed when I tried it, but you may have better luck, and I've been told it worked very well in the past.


# Known issues
While JGT significantly improves the experience in most games, there are still some areas where it has issues.

* JGT replaces English text with number substitutions before sending it to google translate.  Then it replaces the substitutions in the translated text with the original English text.
Unfortunately google sometimes turns the number substitutions into words, like turning the number 1 into the word one, or the number 2 into the word February, which breaks the substitution replacement code.
This is fairly rare, but does happen a lot in game save/load screens where there are a lot of substitutions, some of which are dates.
* The auto translate clipboard feature works great for menus and story dialog, but misses things when the clipboard is changing very rapidly.
This is most noticeable when playing slot machines in Morokuma's text games, or during some combat in Brave of Cloudia.
Hopefully I can improve this in a future version.
For the slot machines specifically, I recommend using Clipboard2Sapi and the ProTalker English synthesizer to hear all the characters in Japanese, especially since some don't have an audible translation in English.

# Closing remarks
The Japanese games are valuable additions to the audio games community and I want to see them continue to be playable by English players.
A big thanks goes out to the developers of those games.
I'd also like to thank the authors of the original instant translate addon who gave me the ability to play the Japanese games in the first place, and whose code was the starting point for this addon.
And BladeStorm360 for writing the original guide for playing Japanese games.
And Tyler Spivey for creating multiple tools to improve the experience when playing Morokuma's text games.
I spent a considerable amount of time and some money to make this addon a reality.
Please do not abuse the service.
I hope everyone enjoys playing the Japanese games as much as I do.

Written by Ian Reed

[1]: https://BlindGamers.com
[2]: https://blindgamers.com/Game/Details/452/Shadow_Rine
[3]: https://blindgamers.com/Game/Details/484/Bokurano_Daiboukenn_3
[4]: http://BlindGamers.com/downloads/JGT.nvda-addon
[5]: http://BlindGamers.com/downloads/JGT_ShadowRine_demo.mp3
[6]: https://blindgamers.com/Game/Details/444/Bokurano_Daibouken_2
[7]: https://blindgamers.com/Game/Details/443/Bokurano_Daibouken_1
[8]: http://BlindGamers.com/downloads/apploc.msi
[9]: http://forum.audiogames.net/viewtopic.php?id=17938
[10]: http://BlindGamers.com/downloads/AlternativeMagicDemo.mp3
[11]: https://dl.dropboxusercontent.com/u/1060978/japaneseGameFix.nvda-addon?dl
[12]: http://BlindGamers.com/downloads/GroundWar1.mp3
[13]: https://xupefei.github.io/Locale-Emulator/
[14]: https://translate.googleusercontent.com/translate_c?act=url&depth=1&hl=en&ie=UTF8&prev=_t&rurl=translate.google.com&sl=ja&tl=en&u=http://www.nyanchangames.com/softs/&usg=ALkJrhjm9Cp68PeUaHCwAnZClbJDlbzpDQ
